- ![Code Einführung Unterprogramme](../assets/code_php.png)
  
  Funktionen sind neben Variablen und Kontrollflussstrukturen der dritte Pfeiler deiner Programmiererausbildung. Mit Hilfe von Funktionen wird dein Code **strukturierter** und **lesbarer**. Und noch wichtiger: Teile des Codes sind **wiederverwendbar**!
- ## Was sind Funktionen?
  
  Stell dir vor, du kannst ein paar Zeilen in ein Paket packen und ein Schild mit **foo** aufs Paket kleben. Jedes Mal, wenn du deine Zeilen im Code benötigst, brauchst du nur noch **foo** hinschreiben. Jenes **foo** ist eine Funktion.
  
  Übrigens hast du Funktionen schon benutzt, in der Form von Befehlen wie `size()`, `ellipse()` und `rect()` sind Funktionen, die das Processing-Team für dich geschrieben hat. In diesem Abschnitt geht es nun darum, selbst solche Befehle bereitzustellen.
- ### Funktionen definieren
  
  Bevor du Funktionen verwenden kannst, musst du Processing sagen, was die Funktion genau tun soll. Das machst du mit der **Definition** der Funktion.
  
  Allgemein definierst du eine (einfache) Funktion wie folgt:
  
  ```Java
  void NAME(){
    CODE
  }
  ```
  
  Wobei CODE für eine beliebige Anzahl an Codezeilen stehen kann. Diese Funktion hier zeichnet ein Gesicht mit zwei schwarzen Knopfaugen an der Stelle (x,y), wobei x und y vorher im Code deklariert sein müssen:
  
  ```Java
  void zeichneGesicht() {
    fill(255); // weiß
    ellipse(x, y, 80, 80); // Gesicht
    fill(0); // schwarz
    ellipse(x-15, y-15, 20, 20); // Auge
    ellipse(x+15, y-15, 20, 20); // Auge
  }
  ```
  
  
  #+BEGIN_IMPORTANT
  Achte darauf, Funktionen **immer klein zu schreiben**. Außerdem können sie nur im aktiven Modus verwendet werden. Etwas merkwürdig sehen noch die runden Klammern hinter dem Funktionsnamen aus, diese bekommen im Abschnitt Parameter Sinn.
  #+END_IMPORTANT
- ### Funktionen aufrufen
  
  Die oben definierte Funktion muss nun noch im "Hauptprogramm" aufgerufen werden. Das vollständige Programm würde wie folgt aussehen:
  
  ```Java
  int x = 50;
  int y = 50;
  
  void setup() { // nichts zu tun
  }
  
  void draw() {
    // sehr übersichtlich :)
    background(255);
    zeichneGesicht(); // Funktionsaufruf
    x++;
  }
  
  // Funktionsdefinition:
  void zeichneGesicht() {
    fill(255);
    ellipse(x, y, 80, 80);
    fill(0);
    ellipse(x-15, y-15, 20, 20);
    ellipse(x+15, y-15, 20, 20);
  }
  ```
  
  Der Vorteil ist hier, dass dein Programm übersichtlicher wird. Wenn man in `draw()` hineinschaut, sieht man direkt, dass offenbar etwas entlang der x-Achse bewegt wird.
- ## Übung
  
  
  #+BEGIN_PINNED
  **Aufgabe 1 - Mauszeiger**
  Schreibe eine Funktion `mauszeiger()`, welche an Position der Maus einen Kreis mit Durchmesser 20 zeichnet.
  #+END_PINNED
  
  ```Java
  Dein Code
  ```
  
  
  #+BEGIN_PINNED
  **Aufgabe 2-  Haus des Nikolaus**
  Erzeuge eine Funktion, die ein Haus des Nikolaus in die Mitte des Fensters zeichnet.
  #+END_PINNED 
  
  
  #+BEGIN_NOTE
  **Zusammenfassung**
  * finde aussagekräftige Funktionsnamen, damit du nicht lang überlegen musst, was das Unterprogramm macht
  * Funktionen können nur bei aktivem Modus verwendet werden
  * Funktionsnamen werden immer klein geschrieben
  * sie machen ein Programm strukturierter und übersichtlicher
  * Fehler müssen nur an einer Stelle korrigiert werden
  * sie ermöglichen neue Programmiermethoden (Rekursion)
  #+END_NOTE
  
  ---
- #+BEGIN_CAUTION
  Wir haben hier nur sehr einfache Funktionen kennengelernt. Funktionen können natürlich auch Werte übergeben bekommen (sogenannte [[Parameter]]) oder auch Werte zurückgeben, mit denen weitergearbeitet werden kann (sogenannte [[Rueckgabewerte]]).
  #+END_CAUTION