### Lernziele


#+BEGIN_QUOTE
* Du kennst die Funktion des Inkrementierens und Dekrementierens.
* Auch die Modulo und Div-Funktion kannst du nun gezielt verwenden.
#+END_QUOTE 


---

Egal ob [[Globale Variablen]] oder [[Lokale Variablen]], du kannst mit beiden eine Menge nützlicher und interessanter Operationen durchführen.

Operatoren sind Symbole, die ein oder zwei Operanden verbinden. Das Plus-Zeichen ist z.B. ein Operator. Die dazugehörig Operation heißt Addition. Hier stellen wir noch einige nützliche Operatoren vor.
- ## Ganzzahlige Division vs. Modulo
  
  Sollten a und b zwei Integer-Zahlen sein, gelten folgende Rechenoperationen:
  
  ```Java
  int a = 13;
  int b = 4;
  
  int c = a / b; // c ist 3 
  int d = a % b; // d ist 1
  
  ```
  
  Die Analogie ist leicht zur Schriftliche Division aus Klasse 4 zu ziehen. Die Operation `/` (ganzzahlige Division) berechnet den ganzzahligen Anteil, die Operation `%` (Modulo) berechnet den Rest.
- ## Inkrement- und Dekrement-Operator
  
  Das bedeutet, dass der Inhalt der Variable x um 1 erhöht wird. Das "++" nennt man einen Operator, und zwar den **Inkrement-Operator**.
  
  ```Java
  x++;
  ```
  
  ... und ist eine Kurzform des Befehls:
  
  ```Java
  x = x + 1;
  ```
  
  Passend dazu gibt es den Operator zum Runterzählen. Dieser wird **Dekrement-Operator** genannt.
  ```Java
  x--;
  ```
  
  
  #+BEGIN_WARNING
  Doch Achtung: Der Inkrement-Operator kann nicht nur allein auf einer Zeile ausgeführt werden, sondern auch in Befehlen oder Rechnungen eingebettet werden:
  #+END_WARNING 
  
  **Beispiel 1**
  ```Java
  int foo = 5;
  println(foo++);
  ```
  
  **Beispiel 2**
  ```Java
  int boo = 10;
  int goo = 100 + boo++;
  ```
  
  Was passiert hier? Wir haben hier zwei Vorgänge, die wir unterscheiden müssen:
  * (a) Verwendung/Substitution der Variable, d.h. die Variable wird durch ihren aktuellen Wert ersetzt
  * (b) Inkrementierung, d.h. der Wert der Variable wird um eins erhöht.
  
  Jetzt wird offenbar, dass es eine große Rolle spielt, ob erst (a) oder (b) durchgeführt wird! Wenn du Beispiel 1 ausführst, siehst du:
  `5`
  
  Der Wert der Variable foo ist aber 6. Das liegt daran, dass beim Einsetzen (a) der alte Wert 5 verwendet wurde. Im Anschluss wurde dann die Variable foo erst um eins auf 6 erhöht.
  
  Verstehst du dann auch Beispiel 2? Probiere es aus und überlege, warum das Ergebnis so ist, wie es ist.
  
  Insbesondere verstehest du jetzt auch, warum der folgende Anfängerfehler so gravierend ist:
  
  ```Java
  int x = 0;
  x = x++;
  ```
  
  Hier passiert folgendes:
  
  1. x wird eingesetzt, d.h. dort steht x = 0, aber dies wird noch nicht umgesetzt
  2. x wird um eins erhöht, d.h. x hat jetzt den Wert eins
  3. die gesamte Zeile, also x = 0, wird jetzt erst ausgeführt, d.h. x wird auf 0 gesetzt
  
  Im Endeffekt bleibt x also immer Null.
  
  #+BEGIN_IMPORTANT
  **Unterschied ++i und i++**
  #+END_IMPORTANT 
  
  Jetzt kann man sich vorstellen, dass diese Reihenfolge umkehren möchte:
  
  (b) Inkrementierung, d.h. der Wert der Variable wird um eins erhöht.
  (a) Verwendung/Substitution der Variable, d.h. die Variable wird durch ihren aktuellen Wert ersetzt
  
  Das heißt, du möchtest erst die Variable erhöhen (b) und dann den neuen Wert einsetzen (a). Dies erreichst du mit der Formulierung:
  ```Java
  ++i;
  ```
  
  Sehen wir uns die Beispiele an, um uns davon zu überzeugen, dass das funktioniert:
  
  **Beispiel 1**
  
  ```Java
  int boo = 10;
  int goo = 100 * ++boo;
  
  print(goo);
  ```
  
  Das Ergebnis lautet ```1100```.
  
  
  **Beispiel 2**
  
  ```Java
  int boo = 10;
  int goo = 100 * boo++;
  
  print(goo);
  ```
  
  Das Ergebnis lautet ```1000```.
  
  
  ---
- ## Übungen
  
  
  #+BEGIN_PINNED
  **Aufgabe 1 - Muster**
  Versuche zu verstehen, wie folgender Code funktioniert. Verändere den Code anschließend so, dass
  a) die Höhe des Dreiecks nur bis zur Hälfte reicht.
  b) nur jede zweite Linie dargestellt wird
  c) das Dreieck von links nach rechts aufgebaut wird - und dabei kleiner wird
  d) einen Kontrastverlauf umsetzt
  #+END_PINNED 
  
  ```Java
  int x = 0;
  int y = 0;
  
  void draw() {
     line(x, 0, x, y);
     x++;
     y++;
  }
  ```
  
  
  #+BEGIN_PINNED
  **Aufgabe 2 - Animation mit Modulo**
  Die Systemvariable `frameCount` zählt von Beginn deines Programms an, wie viele Frames bereits vergangen sind, d.h. die Variable enthält 0, 1, 2, 3 etc.
  Verwende `frameCount` zusammen mit dem Modulo-Operator, um einen Ball immer wieder von links nach rechts fliegen zu lassen. 
  
  #+BEGIN_WARNING
  Dein `draw()` besteht dabei lediglich aus zwei Code-Zeilen.
  #+END_WARNING 
  #+END_PINNED 
  
  ```Java
  Deine Lösung
  ```
  
  #+BEGIN_PINNED
  **Aufgabe 3 - pulsierender Cursor**
  Verwende den Modulo-Trick aus der vorigen Aufgabe, um an die Stelle der Maus einen pulsierenden Cursor(Kreis) zu stellen.
  #+END_PINNED
  
  ```Java
  Deine Lösung
  ```