# Statischer vs. aktiver Modus
- ### Lernziele
  
  
  #+BEGIN_QUOTE
  * Du entscheidest, ob für dein Programm der statische oder aktive Modus das richtige Instrument ist.
  * Du verstehst, warum man beide Modi nicht mischen kann.
  * Du kennst die Möglichkeit der Framerate-Einstellung.
  #+END_QUOTE 
  
  ---
  
  Bisher haben wir stets im sogenannten **statischen Modus** gearbeitet. Dabei sind weder [[Interaktionen]] mit der Maus bzw. Tastatur oder Animationen möglich.
  
  Bringen wir doch etwas **Bewegung** ins Spiel. Und das ist in Processing gar nicht so schwierig. Jedoch muss man erst verstehen, in welche beiden Modi unterschieden wird.
  
  #+BEGIN_NOTE
  **Statischer Modus**
  In Processing können Befehle einfach nacheinander abgearbeitet werden. Dies geschieht zeilenweise und nennt man statischer Modus.
  #+END_NOTE 
  
  
  
  #+BEGIN_NOTE
  **Aktiver Modus**
  Sobald wir Interaktion mit Maus oder Tastatur einbauen möchten, muss der aktive Modus verwendet werden. Dabei werden Grundeinstellungen im `setup()` definiert. Weiter gibt es einen sich dauerhaft wiederholenden Programmabschnitt, welcher im `draw()` definiert wird. Mehr Informationen dazu findest du in diesem Abschnitt.
  #+END_NOTE
- ## Aufbau
  
  Wichtig hierbei zu wissen ist, dass beim aktiven Modus in einer vorgegebenen Wiederholungsrate (FrameRate) ein Bild (Pixelgrafik) erzeugt und dabei bestehende Pixel überschrieben werden.
  
  
  Probier doch einfach mal folgenden Code aus:
  
  ```Java
  void setup() {
     println(“und“);
  }
  
  void draw() {
     println(“hopp“);
  }
  
  ```
  
  Die Funktion `setup()` ist für die Starteinstellungen des Programms zuständig.
  
  Die Funktion `draw()` wird in der eingestellten Wiederholungsrate unendlich oft aufgerufen und ermöglicht somit ein fortlaufendes Programm.
  
  Sobald Processing eine davon erkennt, wird der **aktive Modus** ausgeführt.
  
  Der Inhalt der beiden Funktionen steht im **Code-Block**, das ist der Bereich, der durch die jeweils nachstehenden geschweiften Klammern eingegrenzt ist.
  
  #+BEGIN_TIP
  Das Einrücken des Codes hilft bei der Übersicht des oftmals immer länger werdenden Quelltextes. Da nicht jeder so sorgsam mit dieser Option umgeht, hat Processing Abhilfe geschaffen. Durch das Drücke von **STRG-T** bzw. **CMD-T** wird der Code automatisch eingerückt - Super oder? ;-)
  #+END_TIP
- ## Mischen Impossible ist untersagt!
  
  Man darf aktiven und statischen Modus nicht mischen. Das heißt konkret, sobald `setup()` oder `draw()` erscheint, darf kein Code außerhalb des zugehörigen Code-Blocks geschrieben werden.
- ## Framerate
  
  Die Frage ist, wie oft `draw()` denn nun wiederholt wird. In Anlehnung an Film bzw. Animation spricht man von frame rate, das Aufrufen/Abspielen von draw entspricht einem „frame“. Standardmäßig ist die Wiederholungsrate auf 60 Mal pro Sekunde eingestellt. Diese Wiederholungsrate kann geändert werden:
  
  ```Java
  void setup() {
     frameRate(5);
     println(“und“);
  }
  
  void draw() {
     println(“hopp“);
  }
  
  ```
- ## Übung
- #+BEGIN_PINNED
  **Aufgabe 1**
  Teste den folgenden Code und beschreibe mit eigenen Worten, was geschieht, wenn du den Code ausführst.
  #+END_PINNED
  
  ```Java
  int x = 0;
  int y = 0;
  void setup() {
     frameRate(25);
     background(0);
     size(200, 200);
  }
  
  void draw() {
     background(0);
     fill(255);
     rect(x, y, 50, 50, 5);
     x += 1;
     y += 1;
     if (x>200){
        y = 0;
        x = 0;
     }
  }
  ```
  
  *Lösung*
  ...
  
  
  #+BEGIN_PINNED
  **Aufgabe 2**
  Passe den Code an, sodass sich eine andere Animation ergibt. Ändere z.B. die Richtung und die Form. Kommentiere deinen Code! Füge anschließend deinen neuen Code hier ein.
  #+END_PINNED 
  
  ```Java
  //DEIN KOMMENTIERTER CODE
  ```