- #+BEGIN_IMPORTANT
  Im Falle verschiedener Belegungen für eine Variable kannst du mit der Struktur Switch geeignet auf die jeweiligen Fälle reagieren.
  #+END_IMPORTANT 
  
  ---
  
  Um zu viele Verschachtelungen in einer Verzweigung zu umgehen, gibt es die Möglichkeit verschiedene Zustände für eine Variable mittels `switch` "abzufangen".
- ### Aufbau
  
  ```Java
  ...
  switch (VARIABLE){
  
     case STATUS0: ANWEISUNGEN
     break;
  
     case STATUS1: ANWEISUNGEN
     break;
  
     case STATUS2: ANWEISUNGEN
     break;
  
     default: ANWEISUNGEN
  }
  ...
  
  ```
  
  Mit den Stati sind die möglichen Werte der Variablen gemeint. Default ermöglicht das Abfangen eines alternativen Wertes, welcher nicht in den Stati gelistet ist.
  
  **Beispiel**
  
  ```Java
  int x = 0;
  int speed = 1;
  int state = 0; // 0: play, 1: pause
  
  void draw() {
     background(100);
     noStroke();
     ellipse(x, height/2, 30, 30);
  
     // Was im Zustand "play" gemacht werden soll:
     if (state == 0) {
        x = x + speed;
        if (x < 0 || x > width) {
           speed = -speed;
        }
     }
  }
  
  void keyPressed() {
  
  // NEU: Die switch-Anweisung
  
  switch (state) {
  
     case 0:
        if (key == ' ') {
           state = 1; // Gehe über zu "pause"
        }
        break;
  
     case 1:
        if (keyCode == ENTER) {
           state = 0; // Gehe über zu "play"
        }
        break;
     }
  
     default:
        println("Unbekannter Zustand: " + state);
  }
  ```
  
  ---
- ### Konstanten
  Ein unschöner Aspekt unserer bisherigen Methodik ist die Tatsache, dass Zahlen nicht sehr aussagekräftig sind. Bedeutet 0 jetzt Play oder Pause? Beginnt unsere Zustandszählung bei 1 oder bei 0?
  Am liebsten würden wir unseren Zuständen Namen geben! Wir könnten doch für jeden Zustand eine Variable definieren, die die Zustandszahl enthält und einen aussagekräftigen Namen hat, also zum Beispiel: 
  
  ```Java
  int play = 0;
  int pause = 1;
  ```
  
  Prinzipiell eine gute Idee, aber theoretisch könnte man diese Variablen auch im Programmcode ändern (z.B. indem man play = 42 schreibt). Nun erlaubt Processing/Java, dies zu verbieten, indem man ein `final` vor die Variablendeklaration setzt. Dies ist dann eine **Konstante**. 
  
  
  #+BEGIN_NOTE
  Der Name einer Konstanten wird üblicherweise in GROSSBUCHSTABEN geschrieben. Wenn Ihre Namen aus mehreren Teilwörtern bestehen, verwenden Sie den Unterschrich zur Trennung, z.B. MOVE_UP oder LEVEL_ONE.
  #+END_NOTE  
  
  In unserem Beispiel:
  
  ```Java
  final int Pause = 0;
  final int PLAY = 1;
  ```
  
  Wenn du jetzt versuchst, an einer anderen Stelle im Code die Variable PLAY auf einen neuen Wert zu setzen, bekommst du eine Fehlermeldung, denn Java besteht darauf, dass der Wert konstant bleibt.
  
  ---
- ### Übungen
  
  
  #+BEGIN_PINNED
  **Aufgabe 1**
  Gib einfach "links", "rechts" usw. auf der Konsole aus, wenn die entsprechende Cursortaste (Pfeiltaste) gedrückt wird. Verwende Switch in der keyPressed-Methode. 
  #+END_PINNED 
  
  ```Java
  Dein Code
  ```
  
  #+BEGIN_PINNED
  **Aufgabe 2**
  Lasse einen Kreis von der linken oberen Ecke nach rechts fliegen. Wenn der Kreis die rechte obere Ecke erreicht hat, soll er nach unten fliegen. Wenn er die rechte untere Ecke erreicht hat, nach links. Dann wieder nach oben und von vorn.
  #+END_PINNED 
  
  ```Java
  Dein Code
  ```
  
  #+BEGIN_PINNED
  **Aufgabe 3**
  Du hast schon das Steuern einer Spielfigur implementiert? Da war sicher oft `if` enthalten. Löse dieses Problem mittels `switch`.
  #+END_PINNED 
  
  ```Java
  Dein Code
  ```
  
  #+BEGIN_PINNED
  **Aufgabe 4**
  Schreibe ein einfaches Shooter-Spiel, bei dem sich ein Ball über den Bildschirm bewegt und an den Wänden abprallt. Der Spieler muss mit der Maus auf den Ball klicken, um ihn abzuschießen. Dieses Spiel ist Zustand 1.
  In Zustand 2 bleibt der Ball stehen und vergrößert sich stetig. Der Spieler muss den Ball abschießen bevor er eine bestimmte Größe erreicht, sonst ist das Spiel vorbei (Game Over wäre Zustand 0).
  In Zustand 3 bleibt der Ball ebenfalls stehen und verkleinert sich stetig. Der Spieler muss den Ball abschießen bevor er verschwindet, sonst ist das Spiel vorbei (->Zustand 0)
  
  Dein Spiel sollte in Zustand 1 starten und jedes mal, wenn der Spieler den Ball trifft, **zufällig** in einen anderen Spielzustand wechseln.
  #+END_PINNED
  
  ```Java
  Dein Code
  ```