### Lernziele


#+BEGIN_QUOTE
* Du lernst, wie man eine Variable verwendet.
* Du verstehst, was eine Variable im Arbeitsspeicher erzeugt.
* Du kennst Möglichkeiten für die Verwendung von komplexeren Variablennamen.
#+END_QUOTE 

---
- ## Deklaration
  
  #+BEGIN_NOTE
  Variablen sind kleine Informationsspeicher. Sie können Daten sichern. Dazu muss Speicherplatz freigehalten werden.
  #+END_NOTE 
  
  **Anwendungsbeispiele**
  * zentrale Stelle, wo häufig verwendete Werte abgelegt werden (bspw. Durchmesser von mehreren Kreisen)
  * Wert während des Programmablaufs ändern zu können
  
  
  #+BEGIN_NOTE
  Damit der Computer weiß, wie viel Speicherplatz bereit gestellt werden muss, ist zwingendermaßen der ** [[Datentyp]] ** der zu speichernden Information anzugeben. Weiter muss ein **Variablenname** für den Aufruf des Speicherinhalts definiert werden. Diesen Prozess nennt man **Deklaration einer Variablen**.
  #+END_NOTE 
  
  **Beispiel für Deklarationen von Variablen**
  ```Java
  int zahl;
  String name;
  ```
  
  Allgemein sieht eine Deklaration wie folgt aus:
  
  ```Java
  TYP NAME;
  ```
  
  
  #+BEGIN_WARNING
  Variablennamen immer klein schreiben!
  #+END_WARNING
- ## Initialisierung (Zuweisung)
  
  
  #+BEGIN_NOTE
  Das erste Zuweisen eines Wertes zu einer Variablen wird als **Initialisierung** bezeichnet.
  #+END_NOTE 
  
  ```Java
  zahl = 5;
  name = Manos;
  ```
  
  Auf der rechten Seite kann ein beliebig komplexer arithmetischer Ausdruck stehen:
  (nach bekannten Regeln: Punkt vor Strich, Klammerung)
  
  ```Java
  zahl = (50*14) / 3 + 1;
  ```
  
  
  #+BEGIN_WARNING
  Eine Zuweisung mit Gleichheitszeichen bedeutet nicht mathematische Gleichheit. Stattdessen wird zuerst die rechte Seite ausgewertet und dann das Resultat in die Variable geschrieben.
  #+END_WARNING
  
  
  
  #+BEGIN_IMPORTANT
   Variablen dürfen auch auf der rechten Seite stehen:
  #+END_IMPORTANT
  
  ```Java
  zahl2 = zahl1 + 15;
  ```
- ## Variablenwert verändern
  
  Wie du später merken und anwenden wirst, kommt es sehr häufig vor, das der Wert einer Variable um einen bestimmten Betrag erhöht werden muss.
  
  **Beispiel**
  
  Variable `x` speichert eine x-Koordinate und hat den Wert 10, diesen möchtest du um 5 je draw-Durchlauf erhöhe
  
  ```Java
  x = x + 5;
  ```
  
  
  #+BEGIN_NOTE
  Bei der Veränderung "um 1 erhöhen" spricht man von **Inkrementieren**. Dafür gibt es eine Kurzschreibweise:
  #+END_NOTE 
  
  ```Java
  //statt
  x = x + 1;
  //kann man auch
  x++;
  //schreiben
  ```
  
  
  #+BEGIN_NOTE
  Analog gibt es das **Dekrementieren**, was den Wert eine Variable um eins mindert. Die Kurzschreibweise sieht wie folgt aus:
  #+END_NOTE 
  ```Java
  x--;
  ```
- ## Globale und lokale Variablen
  
  Oft ist es entscheidend, an welcher Stelle des Codes die Variablen deklariert werden. Hierbei unterscheiden wir zwischen [[Globale Variablen]] und [[Lokale Variablen]] sowie [[Systemvariablen]]. 
  ---
- ## Regeln bei der Arbeit mit Variablen
  
  Folgendes musst du beachten:
  
  * `meineVariable` und `meinevariable` sind zwei unterschiedliche Variablen (Groß- und Kleinschreibung beachten)
  * dürfen nicht mit einem Schlüsselwort der Sprache Java identisch sein (bspw. `int` oder `float`)
  * bestehen aus Ziffern und Buchstaben und zwei weiteren Zeichen (Unterstrich und $), die aber besser nicht verwendet werden
  * dürfen nicht mit einer Ziffer beginnen
  * dürfen keine Leerzeichen oder Bindestriche enthalten
  * beginnen mit einem Kleinbuchstaben
  * CamelBack-Regelung: meineNeueVariable
  * sind möglichst aussagekräftig
- ## Übungsaufgaben
  Übungsaufgaben findest du hier: [[Einstiegsaufgaben Variablen]]