### Lernziele

#+BEGIN_QUOTE
Du setzt die Verzweigung ein, um auf verschiedene Fälle bei Entscheidungsfragen zu reagieren.
#+END_QUOTE 

---

Normalerweise wird dein Code Zeile für Zeile, von oben nach unten, ausgeführt. Dies nennt man **Sequenz**. Jedoch geschehen Interaktionen oft nur unter gewissen Voraussetzungen. Deine Spielfigur soll sich nur dann nach links bewegen, wenn eine entsprechende Taste gedrückt wurde.
Dafür gibt es die sogenannte **Verzweigung** (If-Anweisung), die es erlaubt, den **Kontrollfluss** zu verändern.

**Darstellung**

```Java
int punkte = 75;

if (punkte >= 50) {
 println("bestanden");
}
println("tschüs");
```
Ein entsprechendes Flussdiagramm (diese Darstellungsform von Algorithmen solltest du bereits kennen) könnte so aussehen:
![Verzweigung Einführung Flussdiagramm](../assets/verzweigungEinfuehrung.png)
- ## Aufbau einer Verzweigung
  
  ```Java
  if (BEDINGUNG) {
  ANWEISUNG
  }
  ```
  
  
  #+BEGIN_NOTE
  Die sogenannte **Bedingung** ist ein **boolescher Ausdruck**, welche eindeutig wahr oder falsch ist. Hierbei können auch mehrere Aussagen miteinander verknüpft werden (*Ließ dazu den Abschnitt [[Boolesche Ausdruecke]] .*)
  #+END_NOTE 
  
  Auf beiden Seiten des Vergleichs
  * können beliebige komplexe arithmetische Ausdrücke stehen
  * können **Variablen** vorkommen, diese werden bei der Abarbeitung durch ihre aktuellen Werte ersetzt
  
  **Beispiele**`
  * points == 15`
  * `pruef == true`
  * `posX > 15`
  
  
  #+BEGIN_WARNING
  Merke: Der Test auf Gleichheit in Bedingungen erfordert ein doppeltes Gleichheitszeichen ==.
  #+END_WARNING 
  
  **Vergleichsoperatoren**
  * für "kleiner gleich" schreibst du `<=`
  * für "größer gleich" schreibst du `>=`
  * für "ungleich" wird `!=` geschrieben
  
  
  #+BEGIN_TIP
  Die **Anweisung** kann ein einzelner Befehl oder ein ganzer Block von Befehlen sein. In letzterem Fall werden die geschweiften Klammern benötigt. Auch kann in dem Anweisungsblock eine weitere Verzweigung enthalten sein. Dies nennt man dann [[Geschachtelte Verzweigungen]].
  #+END_TIP
- ## If-Then-Else
- ### Aufbau
  
  Soll eine Aktion immer dann geschehen, wenn die Bedingung **nicht** erfüllt ist, können wir noch einen Else-Teil (else heißt sonst) anfügen:
  
  ```Java
  int punkte = 75;
  
  if (punkte >= 50) {
     println("bestanden");
  } else {
     println("durchgefallen");
  }
  println("tschüs");
  ```
  
  ![Verzweigung Einführung Flussdiagramm](../assets/verzweigungElse.png)
- ## Beispiel
  
  Zuerst betrachten wir folgenden Code:
  
  ```Java
  int speed = 1;
  …
  void draw() {
     background(255);
     ellipse(x, 50, 20, 20);
     x = x + speed;
  }
  ```
  
  Doch was wird mit dem Ball passieren, wenn er den Bildrand erreicht hat?
  Genau! Er wird sich weiter in diese Richtung bewegen und nicht mehr im Grafikfenster erscheinen. Das Programm muss um eine Verzweigung erweitert werden. Nur wie lautet diese?
  Wenn der Ball den Rand des Grafikfensters erreicht hat, muss seine Flugrichtung (speed) invertiert (also umgedreht)werden.
  
  ```Java
  int speed = 1;
  …
  void draw() {
     background(255);
     ellipse(x, 50, 20, 20);
     x = x + speed;
     if (x>width) {
        speed = -speed;
     }
  }
  ```
  
  
  #+BEGIN_PINNED
  **Aufgabe**
  Doch reicht das aus? Implementiere den gegebenen Code und vervollständige das Beispiel.
  #+END_PINNED
  
  ```Java
  Deine Lösung
  ```